--[[
	RoundBalerExtension.lua
	
	Author: 	Ifko[nator]
	Date: 		12.08.2020
	Version: 	2.0
	
	Changelog:	v1.0 @24.12.2018 - initial implementation in FS 19
				v1.5 @19.04.2020 - fix for straw harvest addon and the cruise control will not longer set to max, after bale unloading
				v2.0 @12.08.2020 - added support for the fastBale from the Kverneland DLC. The Tractor will no longer stop with tihs baler.
]]

RoundBalerExtension = {};

function RoundBalerExtension.prerequisitesPresent(specializations)
	return true;
end;

function RoundBalerExtension.registerEventListeners(vehicleType)
	local functionNames = {
		"onLoad",
		"onUpdate",
		"onWriteStream",
		"onReadStream"
	};
	
	for _, functionName in ipairs(functionNames) do
		SpecializationUtil.registerEventListener(vehicleType, functionName, RoundBalerExtension);
	end;
end;

function RoundBalerExtension:onLoad(savegame)
	local specRoundBalerExtension = self.spec_roundBalerExtension;
	
	specRoundBalerExtension.hasBaleToUnload = false;
	specRoundBalerExtension.cruiseControlWasOn = false;
	specRoundBalerExtension.hasSetNewTimer = false;
	specRoundBalerExtension.timer = 40;
end;

function RoundBalerExtension:onUpdate(dt, isActiveForInput, isSelected)
	if self:getIsActive() then
		local specAttacherJoints = self.spec_attacherJoints;
		local specRoundBalerExtension = self.spec_roundBalerExtension;
	
		for _, implement in pairs(specAttacherJoints.attachedImplements) do
			if implement ~= nil and implement.object ~= nil then
				local roundBaler = implement.object;
				local controlledTractor = roundBaler:getAttacherVehicle();
				
				local specDrivable = controlledTractor.spec_drivable;
				local specWheelsTractor = controlledTractor.spec_wheels;

				local specBaler = roundBaler.spec_baler;
				local specWheelsRoundBaler = roundBaler.spec_wheels;

				if specBaler ~= nil and specBaler.firstBaleMarker == nil and roundBaler:getIsTurnedOn() and not string.find(roundBaler.configFileName, "fastBale") then
					--## we have an round baler attached :)
					
					if specRoundBalerExtension.hasBaleToUnload then	
						controlledTractor:setBrakeLightsVisibility(true);
					end;
					
					if roundBaler.spec_fillUnit:getFillUnitFillLevel(1) == roundBaler.spec_fillUnit:getFillUnitCapacity(1) then
						if not specRoundBalerExtension.hasBaleToUnload then
							specRoundBalerExtension.hasBaleToUnload = true;
						end;
						
						if specDrivable.cruiseControl.state ~= Drivable.CRUISECONTROL_STATE_OFF then
							specRoundBalerExtension.cruiseControlWasOn = true;
							
							controlledTractor:setCruiseControlState(Drivable.CRUISECONTROL_STATE_OFF);
						end;
						
						if self.isServer then
							for _, wheel in pairs(specWheelsTractor.wheels) do
								setWheelShapeProps(wheel.node, wheel.wheelShape, 0, math.huge, 0, 0);
							end;
							
							for _, wheel in pairs(specWheelsRoundBaler.wheels) do
								setWheelShapeProps(wheel.node, wheel.wheelShape, 0, math.huge, 0, 0);
							end;
						end;
						
						if #specBaler.bales > 0 and specBaler:isUnloadingAllowed() then
							roundBaler:setIsUnloadingBale(true);
						end;
					elseif specBaler.unloadingState ~= Baler.UNLOADING_CLOSED then
						
						local allowClosingBalerChamber = true;

						if roundBaler.getBaleSizeUnit ~= nil then
							local specSHVBB = roundBaler.spec_strawHarvestVariableBaleSizeBaler;
						
							if specSHVBB ~= nil then
								if specSHVBB.currentBaleSizeUnitIndex == 4 and not specRoundBalerExtension.hasSetNewTimer then
									specRoundBalerExtension.timer = 45;

									specRoundBalerExtension.hasSetNewTimer = true;
								end;

								if specRoundBalerExtension.timer > 0 then
									specRoundBalerExtension.timer = specRoundBalerExtension.timer - 1;
								end;

								allowClosingBalerChamber = specRoundBalerExtension.timer == 0;
							end;
						end;

						if specBaler.unloadingState == Baler.UNLOADING_OPEN and allowClosingBalerChamber then
							roundBaler:setIsUnloadingBale(false);
						end;
						
						if self.isServer then
							for _, wheel in pairs(specWheelsTractor.wheels) do
								setWheelShapeProps(wheel.node, wheel.wheelShape, 0, math.huge, 0, 0);
							end;
							
							for _, wheel in pairs(specWheelsRoundBaler.wheels) do
								setWheelShapeProps(wheel.node, wheel.wheelShape, 0, math.huge, 0, 0);
							end;
						end;
					elseif specBaler.unloadingState == Baler.UNLOADING_CLOSED then
						if specRoundBalerExtension.hasBaleToUnload then
							if specRoundBalerExtension.cruiseControlWasOn then	
								controlledTractor:setCruiseControlState(Drivable.CRUISECONTROL_STATE_ACTIVE);
								
								specRoundBalerExtension.cruiseControlWasOn = false;
							end;

							if roundBaler.getBaleSizeUnit ~= nil then
								specRoundBalerExtension.timer = 40;
								specRoundBalerExtension.hasSetNewTimer = false;
							end;
							
							specRoundBalerExtension.hasBaleToUnload = false;
						end;
					end;
				end;
			end;
		end;
	end;
end;

function RoundBalerExtension:onWriteStream(streamId, connection)
    if not connection:getIsServer() then 
		local specRoundBalerExtension = self.spec_roundBalerExtension;
		
		streamWriteInt8(streamId, specRoundBalerExtension.timer);
	end;
end;

function RoundBalerExtension:onReadStream(streamId, connection)
    if connection:getIsServer() then
		local specRoundBalerExtension = self.spec_roundBalerExtension;
		
		specRoundBalerExtension.timer = streamReadInt8(streamId);
	end;
end;